/**
 * Copyright 2019-2021 Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of DesQ Shell (https://gitlab.com/DesQ/Shell)
 * DesQ Shell is the main Shell UI of the DesQ Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#include "DesQWidget.hpp"

#include <desq/Utils.hpp>
#include <desq/desq-config.h>

// DesQ Headers
#include <desqui/ShellPlugin.hpp>

DesQWidget::DesQWidget( QString widget ) {
    QSettings wSett( MetadataPath + widget + ".ini", QSettings::IniFormat );

    mPluginName = widget;
    mWidgetName = wSett.value( "Name" ).toString();
    mWidgetDesc = wSett.value( "Descrpition" ).toString();

    setToolTip( mWidgetDesc );
    setAttribute( Qt::WA_TranslucentBackground );

    setFixedWidth( 200 );

    /* To show the widget name and the error message */
    setMinimumHeight( 50 );

    QTimer::singleShot( 1000, this, &DesQWidget::prepareWidget );
}


void DesQWidget::prepareWidget() {
    mState = State::Loading;
    repaint();

    QHBoxLayout *lyt = new QHBoxLayout();

    lyt->setContentsMargins( QMargins( 2, 20, 2, 2 ) );

    QPluginLoader loader( PluginPath "shell/libdesq-plugin-" + mPluginName + ".so" );
    QObject       *pObject = loader.instance();

    QWidget *widget = nullptr;

    if ( pObject ) {
        DesQ::Plugin::Shell *plugin = qobject_cast<DesQ::Plugin::Shell *>( pObject );

        if ( !plugin ) {
            qWarning() << "Plugin Error:" << loader.errorString();
        }

        else {
            widget = plugin->widget( this );
        }
    }

    else {
        qWarning() << "Plugin Error:" << loader.errorString();
    }

    if ( widget ) {
        mState = State::Loaded;
        lyt->addWidget( widget );
        setLayout( lyt );
    }

    else {
        mState = State::Failed;
    }

    repaint();
}


void DesQWidget::resizeContainer() {
    qDebug() << "Not implemented";
}


void DesQWidget::paintEvent( QPaintEvent *pEvent ) {
    QPainter painter( this );

    painter.setRenderHints( QPainter::Antialiasing );

    QColor brush( palette().color( QPalette::Window ) );

    brush.setAlphaF( 0.2 );

    painter.save();
    painter.setPen( Qt::NoPen );
    painter.setBrush( brush );
    painter.drawRoundedRect( rect().adjusted( 0, 0, -1, -1 ), 3, 3 );
    painter.restore();

    switch ( mState ) {
        case State::NotLoaded: {
            painter.drawText( rect(), Qt::AlignCenter | Qt::TextWordWrap, "Waiting for widget..." );
            pEvent->accept();
            break;
        }

        case State::Loading: {
            painter.drawText( rect(), Qt::AlignCenter | Qt::TextWordWrap, "Loading the widget..." );
            pEvent->accept();
            break;
        }

        case State::Loaded: {
            /* Nothing from our side */
            painter.drawText( QRect( 0, 3, width(), height() ), Qt::AlignTop | Qt::AlignHCenter | Qt::TextWordWrap, mWidgetName );
            QWidget::paintEvent( pEvent );
            break;
        }

        case State::Failed: {
            painter.drawText( rect(), Qt::AlignCenter | Qt::TextWordWrap, "Failed to load the widget" );
            pEvent->accept();
            break;
        }
    }

    painter.end();
}
